function lo = loewner_order(X,Y)

% Loewner order of (1x1,) 2x2 or 3x3 Hermitian matrices X and Y: is X > Y?
%
% Loewner order: X > Y if (X-Y) is positive definite
%
% lo = loewner_order(X,Y)
%
% SAR input images are images of the (row-wise) upper-right elements of
% the covariance or coherency matrix, e.g. for full polarimetry:
% a vector with nine real (i.e., non-complex) elements, namely
% [ShhShh* Re(ShhShv*) Im(ShhShv*) Re(ShhSvv*) Im(ShhSvv*) ...
%  ShvShv* Re(ShvSvv*) Im(ShvSvv*) SvvSvv*]
%
% Input
%   X       - 3-D array of (1x1,) 2x2 or 3x3 complex covariance matrix
%             (polarimetric SAR image in covariance matrix form,
%             reading order is alphabetical (as read by freadenvisar):
%               hhhh (real), hhhv (complex), hhvv (complex),
%                            hvhv (real),    hvvv (complex),
%                                            vvvv (real)
%   Y       - 3-D array of (1x1,) 2x2 or 3x3 complex covariance matrix
%
% Output
%   lo - struct with fields
%           eigvals	- eigenvalues of X-Y
%           posdef	- X-Y is positive definite (logical)
%           negdef	- X-Y is negative definite (logical)
%
% Reference
%
% Allan A. Nielsen, Henning Skriver and Knut Conradsen (2019).
% "The Loewner Order Applied to Change Detection in Sentinel-1 and
% Radarsat-2 Data".
% Early Access IEEE Geoscience and Remote Sensing Letters, June 2019
% DOI:10.1109/LGRS.2019.2918636

% (c) Copyright 2018-2019
% Allan Aasbjerg Nielsen, PhD, MSc
% alan@dtu.dk, people.compute.dtu.dk/alan
% 3-21 Sep 2018
% 13 Apr 2019 (fast, matrix based 2x2 and 3x3 azimuthal symmetry)
% 6-9 Jul 2019 (fast, matrix based full 3x3)

if nargin<2
    help loewner_order
    error('loewner_order: wrong input');
end
if nargout>1
    help loewner_order
    error('loewner_order: wrong output');
end

if ischar(X), X = freadenvisar(X); end
if ischar(Y), Y = freadenvisar(Y); end

pX = size(X);
pY = size(Y);
if ~all(pX==pY)
    error('loewner_order: X and Y must have same dimensions');
end
nrows = pX(1);
ncols = pX(2);
if  ndims(X)==3
    nvars = pX(3);
elseif ismatrix(X)
    warning('scalar input, no reason to use loewner_order')
    nvars = 1;
else
    error('loewner_order: wrong sizes of input images');
end

if nvars==9 % full pol
%     fprintf('loewner_order: full pol\n')
    p = 3;
elseif nvars==4 % dual pol
%     fprintf('loewner_order: dual pol\n')
    p = 2;
elseif nvars==2 % dual pol, diagonal only
%     fprintf('loewner_order: dual pol, diagonal only\n')
    p = 2;
elseif nvars==1 % single channel
%     fprintf('loewner_order: single pol - makes no sense\n')
%     return
    p = 1;
elseif nvars==5 % full pol, azimuthal symmetry
%     fprintf('loewner_order: full pol, azimuthal symmetry\n')
    p = 3;
elseif nvars==3 % full pol, diagonal only
%     fprintf('loewner_order: full pol, diagonal only\n')    
    p = 3;
else
    error('loewner_order: wrong dimensionality of input images')
end

lo.eigvals = nan(nrows,ncols,p);

if nvars==2 || nvars==3 || nvars==1
    X = X-Y;
    clear Y
    lo.eigvals = sort(X,3,'descend');
    clear X
elseif nvars==4 % dual pol
    X = X-Y;
    clear Y
    disc = sqrt((X(:,:,1)-X(:,:,4)).^2+4*(X(:,:,2).^2+X(:,:,3).^2));
    lo.eigvals(:,:,1) = 0.5*(X(:,:,1)+X(:,:,4) + disc);
%     lo.eigvals(:,:,2) = 0.5*(X(:,:,1)+X(:,:,4) - disc);
    clear X
    lo.eigvals(:,:,2) = lo.eigvals(:,:,1) - disc;
    clear disc
elseif nvars==5 % azimuthal symmetry
    X = X-Y;
    clear Y
    disc = sqrt((X(:,:,1)-X(:,:,5)).^2+4*(X(:,:,2).^2+X(:,:,3).^2));
    lo.eigvals(:,:,1) = X(:,:,4);
    lo.eigvals(:,:,2) = 0.5*(X(:,:,1)+X(:,:,5) + disc);
%     lo.eigvals(:,:,3) = 0.5*(X(:,:,1)+X(:,:,5) - disc);
    clear X
    lo.eigvals(:,:,3) = lo.eigvals(:,:,2) - disc;
    clear disc
    lo.eigvals = sort(lo.eigvals,3,'descend'); % to be on the safe side
else % full pol
    X = X-Y;
    clear Y
	a2 = X(:,:,2).^2+X(:,:,3).^2;
	rho2 = X(:,:,4).^2+X(:,:,5).^2;
	b2 = X(:,:,7).^2+X(:,:,8).^2;
	% minus determinant
    D = -(X(:,:,6).*(X(:,:,1).*X(:,:,9)-rho2)-X(:,:,1).*b2-X(:,:,9).*a2 + ...
		2.0*(X(:,:,4).*(X(:,:,2).*X(:,:,7)-X(:,:,3).*X(:,:,8)) + ...
             X(:,:,5).*(X(:,:,3).*X(:,:,7)+X(:,:,2).*X(:,:,8))));
	C = X(:,:,6).*(X(:,:,1)+X(:,:,9))+X(:,:,1).*X(:,:,9)-rho2-a2-b2;
    clear a2 rho2 b2
	% minus trace
    B = -(X(:,:,1)+X(:,:,6)+X(:,:,9));
    clear X
    % A = 1;
% 	Q = (2.0/27.0*(B./A).^3-1.0/3.0*B.*C./A.^2+D./A)/2.0;
	Q = (2.0/27.0*B.^3-1.0/3.0*B.*C+D)/2.0;
    clear D
%   P = (-1.0/3.0*(B./A).^2+C./A)/3.0;
    P = (-1.0/3.0*B.^2+C)/3.0;
    clear C
    P3 = P.^3;
    Q2P3 = Q.^2+P3;
    if (Q2P3>0) % negation of casus irreducibilis
        imtool(Q2P3)
    end
    clear Q2P3
    % casus irreducibilis: Q2P3 must be non-positive
    phi = acos(-Q./sqrt(-P3));
    clear Q P3
    tsmp = 2.0*sqrt(-P);
    clear P
	lo.eigvals(:,:,1) =  tsmp.*cos( phi    /3.0);
	lo.eigvals(:,:,2) = -tsmp.*cos((phi+pi)/3.0);
% 	lo.eigvals(:,:,3) = -tsmp.*cos((phi-pi)/3.0);
    clear phi tsmp
	lo.eigvals(:,:,3) = -sum(lo.eigvals(:,:,1:2),3);
    lo.eigvals = lo.eigvals - B/3.0;
    clear B
%     lo.eigvals = sort(lo.eigvals,3,'descend');
end
lo.posdef = lo.eigvals(:,:,p) > 0;
lo.negdef = lo.eigvals(:,:,1) < 0;
% possible remaining obs are indefinite: ~(lo.posdef | lo.negdef)
